@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2013, 2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2013-2014                                Inria
+ * Copyright (C) 2010-2016                                CNRS
+ * Copyright (C) 2009-2010,2013-2015                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -33,12 +34,14 @@ void cpu_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 	int *val = (int *)STARPU_VARIABLE_GET_PTR(descr[0]);
 
 	*val += 1;
+	printf("task executing \n");
 }
 
 struct starpu_codelet cl =
 {
 	.modes = { STARPU_RW },
 	.cpu_funcs = {cpu_codelet},
+	.cpu_funcs_name = {"cpu_codelet"},
 	.nbuffers = 1,
 	.name = "callback"
 };
@@ -50,6 +53,13 @@ void prologue_callback_func(void *callback_arg)
 	STARPU_ASSERT(*x == -999.0);
 }
 
+void pop_prologue_callback_func(void *args)
+{
+	unsigned val = (uintptr_t) args;
+	printf("pop_prologue_callback val %u \n", val);
+	STARPU_ASSERT(val == 5);
+}
+
 
 int main(int argc, char **argv)
 {
@@ -61,37 +71,38 @@ int main(int argc, char **argv)
 		return 77;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	starpu_variable_data_register(&handle, 0, (uintptr_t)&v, sizeof(int));
+	starpu_variable_data_register(&handle, STARPU_MAIN_RAM, (uintptr_t)&v, sizeof(int));
 	double x = -999.0;
 
 	struct starpu_task *task = starpu_task_create();
 	task->cl = &cl;
 	task->prologue_callback_func = prologue_callback_func;
 	task->prologue_callback_arg = &x;
+
+	task->prologue_callback_pop_func = pop_prologue_callback_func;
+	task->prologue_callback_pop_arg = (void*) 5;
+
 	task->handles[0] = handle;
 
 	ret = starpu_task_submit(task);
 	if (ret == -ENODEV) goto enodev;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 
-	int ret2 = starpu_insert_task(&cl,
-				      STARPU_RW, handle,
-				      STARPU_PROLOGUE_CALLBACK, prologue_callback_func,
-				      STARPU_PROLOGUE_CALLBACK_ARG, &x,
-				      0);
-
+	ret = starpu_task_insert(&cl,
+				 STARPU_RW, handle,
+				 STARPU_PROLOGUE_CALLBACK, prologue_callback_func,
+				 STARPU_PROLOGUE_CALLBACK_ARG, &x,
+				 STARPU_PROLOGUE_CALLBACK_POP, pop_prologue_callback_func,
+				 STARPU_PROLOGUE_CALLBACK_POP_ARG, 5,
+				 0);
+	if (ret == -ENODEV) goto enodev;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 
 	starpu_task_wait_for_all();
-	starpu_data_unregister(handle);
-
-	FPRINTF(stderr, "v -> %d\n", v);
-
-
-	starpu_shutdown();
-
-	return 0;
 
 enodev:
+	starpu_data_unregister(handle);
+	FPRINTF(stderr, "v -> %d\n", v);
 	starpu_shutdown();
-	return 77;
+	return (ret == -ENODEV) ? 77 : 0;
 }
