@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2013, 2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2009-2011,2013-2015                      Université de Bordeaux
+ * Copyright (C) 2010-2013,2015-2016                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -92,7 +93,7 @@ static void create_task_grid(unsigned iter)
 		struct starpu_task *task = tasks[iter][i] = starpu_task_create();
 
 		task->cl = &cl;
-		task->cl_arg = (void*)(uintptr_t) (i | (iter << 16));
+		/* task->cl_arg = (void*)(uintptr_t) (i | (iter << 16)); */
 
 		task->use_tag = 1;
 		task->tag_id = TAG(i, iter);
@@ -109,11 +110,12 @@ static void create_task_grid(unsigned iter)
 static int start_task_grid(unsigned iter)
 {
 	unsigned i;
-	int ret;
 
 	/* FPRINTF(stderr, "start grid %d ni %d...\n", iter, ni); */
 
-	for (i = 0; i < ni; i++) {
+	for (i = 0; i < ni; i++)
+	{
+		int ret;
 		ret = starpu_task_submit(tasks[iter][i]);
 		if (ret == -ENODEV) return 77;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
@@ -149,6 +151,7 @@ int main(int argc STARPU_ATTRIBUTE_UNUSED , char **argv STARPU_ATTRIBUTE_UNUSED)
 
 	starpu_codelet_init(&cl);
 	cl.cpu_funcs[0] = cpu_codelet;
+	cl.cpu_funcs_name[0] = "cpu_codelet";
 	cl.cuda_funcs[0] = cpu_codelet;
 	cl.opencl_funcs[0] = cpu_codelet;
 	cl.nbuffers = 0;
@@ -177,7 +180,8 @@ int main(int argc STARPU_ATTRIBUTE_UNUSED , char **argv STARPU_ATTRIBUTE_UNUSED)
 	FPRINTF(stderr, "TEST DONE ...\n");
 
 enodev:
-	for (i = 0; i < Nrolls; i++) {
+	for (i = 0; i < Nrolls; i++)
+	{
 		for (j = 0; j < ni; j++)
 			starpu_task_destroy(tasks[i][j]);
 		free(tasks[i]);
