@@ -1,9 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2017  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2017  CNRS
- * Copyright (C) 2011  Télécom-SudParis
- * Copyright (C) 2011, 2014  INRIA
+ * Copyright (C) 2011-2014                                Inria
+ * Copyright (C) 2009-2018                                Université de Bordeaux
+ * Copyright (C) 2010-2015,2017                           CNRS
+ * Copyright (C) 2011                                     Télécom-SudParis
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -121,6 +121,7 @@ struct starpu_codelet
 	unsigned long per_worker_stats[STARPU_NMAXWORKERS];
 
 	const char *name;
+	unsigned color;
 
 	int flags;
 };
@@ -172,10 +173,11 @@ struct starpu_task
 	unsigned destroy:1;
 	unsigned regenerate:1;
 
-	unsigned scheduled:1;
-
 	unsigned int mf_skip:1;
 
+	unsigned scheduled:1;
+	unsigned prefetched:1;
+
 	unsigned workerid;
 	unsigned workerorder;
 
@@ -184,6 +186,8 @@ struct starpu_task
 	enum starpu_task_status status;
 
 	int magic;
+	unsigned type;
+	unsigned color;
 
 	unsigned sched_ctx;
 	int hypervisor_tag;
@@ -200,7 +204,6 @@ struct starpu_task
 	struct starpu_task *prev;
 	struct starpu_task *next;
 	void *starpu_private;
-	unsigned prefetched;
 #ifdef STARPU_OPENMP
 	struct starpu_omp_task *omp_task;
 #else
@@ -208,6 +211,10 @@ struct starpu_task
 #endif
 };
 
+#define STARPU_TASK_TYPE_NORMAL		0
+#define STARPU_TASK_TYPE_INTERNAL	(1<<0)
+#define STARPU_TASK_TYPE_DATA_ACQUIRE	(1<<1)
+
 /* Note: remember to update starpu_task_init as well */
 #define STARPU_TASK_INITIALIZER 			\
 {							\
@@ -232,6 +239,8 @@ struct starpu_task
 	.predicted_transfer = NAN,			\
 	.starpu_private = NULL,				\
 	.magic = 42,                  			\
+	.type = 0,					\
+	.color = 0,					\
 	.sched_ctx = STARPU_NMAX_SCHED_CTXS,		\
 	.hypervisor_tag = 0,				\
 	.flops = 0.0,					\
