@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2013,2015                           CNRS
+ * Copyright (C) 2009-2010,2014                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,8 +25,8 @@ extern unsigned nblocks;
 extern unsigned nbigblocks;
 extern unsigned noprio;
 extern unsigned display;
-extern unsigned dblockx;
-extern unsigned dblocky;
+extern int dblockx;
+extern int dblocky;
 
 void parse_args(int argc, char **argv, int nodes);
 
