@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011, 2014, 2017  Université de Bordeaux
- * Copyright (C) 2010, 2012, 2013, 2017  CNRS
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2010-2011,2014,2017                      Université de Bordeaux
+ * Copyright (C) 2010-2013,2015,2017,2018                 CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -847,7 +848,7 @@ static void wait_termination(void)
  *	code to bootstrap the factorization
  */
 
-double STARPU_PLU(plu_main)(unsigned _nblocks, int _rank, int _world_size)
+double STARPU_PLU(plu_main)(unsigned _nblocks, int _rank, int _world_size, unsigned _no_prio)
 {
 	double start;
 	double end;
@@ -855,6 +856,7 @@ double STARPU_PLU(plu_main)(unsigned _nblocks, int _rank, int _world_size)
 	nblocks = _nblocks;
 	rank = _rank;
 	world_size = _world_size;
+	no_prio = _no_prio;
 
 	/* create all the DAG nodes */
 	unsigned i,j,k;
