@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2011                                     Inria
+ * Copyright (C) 2012,2015,2017                           CNRS
+ * Copyright (C) 2010-2011, 2018                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,15 +18,16 @@
 
 #include "socl.h"
 
+CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_mem CL_API_CALL
 soclCreateImage2D(cl_context              UNUSED(context),
-                cl_mem_flags            UNUSED(flags),
-                const cl_image_format * UNUSED(image_format),
-                size_t                  UNUSED(image_width),
-                size_t                  UNUSED(image_height),
-                size_t                  UNUSED(image_row_pitch), 
-                void *                  UNUSED(host_ptr),
-                cl_int *                errcode_ret) CL_API_SUFFIX__VERSION_1_0
+		  cl_mem_flags            UNUSED(flags),
+		  const cl_image_format * UNUSED(image_format),
+		  size_t                  UNUSED(image_width),
+		  size_t                  UNUSED(image_height),
+		  size_t                  UNUSED(image_row_pitch),
+		  void *                  UNUSED(host_ptr),
+		  cl_int *                errcode_ret)
 {
    if (errcode_ret != NULL)
       *errcode_ret = CL_INVALID_OPERATION;
