@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2011-2012                                Inria
+ * Copyright (C) 2012,2015,2017                           CNRS
+ * Copyright (C) 2010-2011, 2018                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,12 +18,20 @@
 
 #include "socl.h"
 
+CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_int CL_API_CALL
-soclEnqueueBarrier(cl_command_queue cq) CL_API_SUFFIX__VERSION_1_0
+soclEnqueueBarrier(cl_command_queue cq)
 {
-	command_marker cmd = command_barrier_create();
+	command_barrier cmd = command_barrier_create();
 
 	command_queue_enqueue(cq, cmd, 0, NULL);
 
 	return CL_SUCCESS;
 }
+
+cl_int command_barrier_submit(command_barrier cmd) {
+	struct starpu_task *task;
+	task = task_create(CL_COMMAND_BARRIER);
+
+	return task_submit(task, cmd);
+}
