@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2011                                     Inria
+ * Copyright (C) 2012,2015,2017                           CNRS
+ * Copyright (C) 2010-2011, 2018                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,17 +18,18 @@
 
 #include "socl.h"
 
+CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_int CL_API_CALL
 soclEnqueueNativeKernel(cl_command_queue  UNUSED(command_queue),
-					       __attribute__((unused)) void (*user_func)(void *), 
-                      void *            UNUSED(args),
-                      size_t            UNUSED(cb_args), 
-                      cl_uint           UNUSED(num_mem_objects),
-                      const cl_mem *    UNUSED(mem_list),
-                      const void **     UNUSED(args_mem_loc),
-                      cl_uint           UNUSED(num_events_in_wait_list),
-                      const cl_event *  UNUSED(event_wait_list),
-                      cl_event *        UNUSED(event)) CL_API_SUFFIX__VERSION_1_0
+			__attribute__((unused)) void (*user_func)(void *),
+			void *            UNUSED(args),
+			size_t            UNUSED(cb_args),
+			cl_uint           UNUSED(num_mem_objects),
+			const cl_mem *    UNUSED(mem_list),
+			const void **     UNUSED(args_mem_loc),
+			cl_uint           UNUSED(num_events_in_wait_list),
+			const cl_event *  UNUSED(event_wait_list),
+			cl_event *        UNUSED(event))
 {
    return CL_INVALID_OPERATION;
 }
