@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2011  Université de Bordeaux
- * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2011                                     Inria
+ * Copyright (C) 2008-2011,2014                           Université de Bordeaux
+ * Copyright (C) 2010-2012,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -163,11 +164,11 @@ int _starpu_regression_non_linear_power(struct starpu_perfmodel_history_list *pt
 {
 	unsigned n = find_list_size(ptr);
 
-	unsigned *x = (unsigned *) malloc(n*sizeof(unsigned));
-	STARPU_ASSERT(x);
+	unsigned *x;
+	_STARPU_MALLOC(x, n*sizeof(unsigned));
 
-	double *y = (double *) malloc(n*sizeof(double));
-	STARPU_ASSERT(y);
+	double *y;
+	_STARPU_MALLOC(y, n*sizeof(double));
 
 	dump_list(x, y, ptr);
 
