@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011, 2013  Université de Bordeaux
- * Copyright (C) 2011  Télécom-SudParis
- * Copyright (C) 2012  Inria
+ * Copyright (C) 2011-2012                                Inria
+ * Copyright (C) 2011-2014                                Université de Bordeaux
+ * Copyright (C) 2011,2013,2015,2017                      CNRS
+ * Copyright (C) 2011                                     Télécom-SudParis
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,8 +29,7 @@
 /* Initialize a task bundle */
 void starpu_task_bundle_create(starpu_task_bundle_t *bundle)
 {
-	*bundle = (starpu_task_bundle_t) malloc(sizeof(struct _starpu_task_bundle));
-	STARPU_ASSERT(*bundle);
+	_STARPU_MALLOC(*bundle, sizeof(struct _starpu_task_bundle));
 
 	STARPU_PTHREAD_MUTEX_INIT(&(*bundle)->mutex, NULL);
 	/* Of course at the beginning a bundle is open,
@@ -62,8 +62,8 @@ int starpu_task_bundle_insert(starpu_task_bundle_t bundle, struct starpu_task *t
 
 	/* Insert a task at the end of the bundle */
 	struct _starpu_task_bundle_entry *entry;
-	entry = (struct _starpu_task_bundle_entry *) malloc(sizeof(struct _starpu_task_bundle_entry));
-	STARPU_ASSERT(entry);
+	_STARPU_MALLOC(entry, sizeof(struct _starpu_task_bundle_entry));
+
 	entry->task = task;
 	entry->next = NULL;
 
@@ -198,8 +198,8 @@ void _insertion_handle_sorted(struct _starpu_handle_list **listp, starpu_data_ha
 	 * list, we insert it as first element */
 	if (!list || list->handle > handle)
 	{
-		struct _starpu_handle_list *link = (struct _starpu_handle_list *) malloc(sizeof(struct _starpu_handle_list));
-		STARPU_ASSERT(link);
+		struct _starpu_handle_list *link;
+		_STARPU_MALLOC(link, sizeof(struct _starpu_handle_list));
 		link->handle = handle;
 		link->mode = mode;
 		link->next = list;
@@ -226,8 +226,8 @@ void _insertion_handle_sorted(struct _starpu_handle_list **listp, starpu_data_ha
 	{
 		/* The handle was not in the list, we insert it after 'prev', thus right before
 		 * 'list' which is the smallest following handle */
-		struct _starpu_handle_list *link = (struct _starpu_handle_list *) malloc(sizeof(struct _starpu_handle_list));
-		STARPU_ASSERT(link);
+		struct _starpu_handle_list *link;
+		_STARPU_MALLOC(link, sizeof(struct _starpu_handle_list));
 		link->handle = handle;
 		link->mode = mode;
 		link->next = prev->next;
