@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015  Centre National de la Recherche Scientifique
+ * Copyright (C) 2011,2014-2016                           Inria
+ * Copyright (C) 2008-2016                                Université de Bordeaux
+ * Copyright (C) 2010-2015,2017                           CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,7 +28,6 @@
 #include <common/fxt.h>
 #include <common/list.h>
 
-#include <datawizard/data_request.h>
 #include <datawizard/interfaces/data_interface.h>
 #include <datawizard/datastats.h>
 #include <datawizard/memstats.h>
@@ -41,18 +41,20 @@ enum _starpu_cache_state
 };
 
 /* this should contain the information relative to a given data replicate  */
-struct _starpu_data_replicate {
+struct _starpu_data_replicate
+{
 	starpu_data_handle_t handle;
 
 	/* describe the actual data layout, as manipulated by data interfaces in *_interface.c */
 	void *data_interface;
 
-	unsigned memory_node;
+	/* How many requests or tasks are currently working with this replicate */
+	int refcnt;
+
+	char memory_node;
 
 	/* describes the state of the local data in term of coherency */
-	enum _starpu_cache_state	state;
-
-	int refcnt;
+	enum _starpu_cache_state	state: 2;
 
 	/* A buffer that is used for SCRATCH or reduction cannnot be used with
 	 * filters. */
@@ -70,9 +72,6 @@ struct _starpu_data_replicate {
 	 * */
 	unsigned automatically_allocated:1;
 
-        /* Pointer to memchunk for LRU strategy */
-	struct _starpu_mem_chunk * mc;
-
 	/* To help the scheduling policies to make some decision, we
 	   may keep a track of the tasks that are likely to request
 	   this data on the current node.
@@ -82,6 +81,9 @@ struct _starpu_data_replicate {
 	 */
 	uint32_t requested;
 	struct _starpu_data_request *request[STARPU_MAXNODES];
+
+        /* Pointer to memchunk for LRU strategy */
+	struct _starpu_mem_chunk * mc;
 };
 
 struct _starpu_data_requester_list;
@@ -100,7 +102,8 @@ struct _starpu_task_wrapper_list
 };
 
 /* This structure describes a doubly-linked list of task */
-struct _starpu_task_wrapper_dlist {
+struct _starpu_task_wrapper_dlist
+{
 	struct starpu_task *task;
 	struct _starpu_task_wrapper_dlist *next;
 	struct _starpu_task_wrapper_dlist *prev;
@@ -110,13 +113,20 @@ extern int _starpu_has_not_important_data;
 
 typedef void (*_starpu_data_handle_unregister_hook)(starpu_data_handle_t);
 
+/* This is initialized in both _starpu_register_new_data and _starpu_data_partition */
 struct _starpu_data_state
 {
+	int magic;
 	struct _starpu_data_requester_list req_list;
 	/* the number of requests currently in the scheduling engine (not in
 	 * the req_list anymore), i.e. the number of holders of the
 	 * current_mode rwlock */
 	unsigned refcnt;
+	/* whether we are already unlocking data requests */
+	unsigned unlocking_reqs;
+	/* Current access mode. Is always either STARPU_R, STARPU_W,
+	 * STARPU_SCRATCH or STARPU_REDUX, but never a combination such as
+	 * STARPU_RW. */
 	enum starpu_data_access_mode current_mode;
 	/* protect meta data */
 	struct _starpu_spinlock header_lock;
@@ -139,31 +149,45 @@ struct _starpu_data_state
 
 	starpu_data_handle_t children;
 	unsigned nchildren;
+	/* How many partition plans this handle has */
+	unsigned nplans;
+	/* Switch codelet for asynchronous partitioning */
+	struct starpu_codelet *switch_cl;
+	/* size of dyn_nodes recorded in switch_cl */
+	unsigned switch_cl_nparts;
+	/* Whether a partition plan is currently submitted and the
+	 * corresponding unpartition has not been yet
+	 *
+	 * Or the number of partition plans currently submitted in readonly
+	 * mode.
+	 */
+	unsigned partitioned;
+	/* Whether a partition plan is currently submitted in readonly mode */
+	unsigned readonly;
 
 	/* describe the state of the data in term of coherency */
 	struct _starpu_data_replicate per_node[STARPU_MAXNODES];
-	struct _starpu_data_replicate per_worker[STARPU_NMAXWORKERS];
+	struct _starpu_data_replicate *per_worker;
 
 	struct starpu_data_interface_ops *ops;
 
 	/* Footprint which identifies data layout */
 	uint32_t footprint;
 
-	/* where is the data home ? -1 if none yet */
+	/* where is the data home, i.e. which node it was registered from ? -1 if none yet */
 	int home_node;
 
 	/* what is the default write-through mask for that data ? */
 	uint32_t wt_mask;
 
-	/* allows special optimization */
-	uint8_t is_readonly;
-
 	/* in some case, the application may explicitly tell StarPU that a
  	 * piece of data is not likely to be used soon again */
 	unsigned is_not_important;
 
 	/* Does StarPU have to enforce some implicit data-dependencies ? */
 	unsigned sequential_consistency;
+	/* Is the data initialized, or a task is already submitted to initialize it */
+	unsigned initialized;
 
 	/* This lock should protect any operation to enforce
 	 * sequential_consistency */
@@ -214,8 +238,15 @@ struct _starpu_data_state
 
 	starpu_data_handle_t *reduction_tmp_handles;
 
+	/* Final request for write invalidation */
+	struct _starpu_data_request *write_invalidation_req;
+
 	unsigned lazy_unregister;
 
+#ifdef STARPU_OPENMP
+	unsigned removed_from_context_hash;
+#endif
+
         /* Used for MPI */
 	void *mpi_data;
 
@@ -225,9 +256,20 @@ struct _starpu_data_state
 
 	/* hook to be called when unregistering the data */
 	_starpu_data_handle_unregister_hook unregister_hook;
-};
 
-void _starpu_display_msi_stats(void);
+	struct starpu_arbiter *arbiter;
+	/* This is protected by the arbiter mutex */
+	struct _starpu_data_requester_list arbitered_req_list;
+
+	/* Data maintained by schedulers themselves */
+	/* Last worker that took this data in locality mode, or -1 if nobody
+	 * took it yet */
+	int last_locality;
+
+	/* A generic pointer to data in the user land (could be anything and this
+	 * is not manage by StarPU) */
+	void *user_data;
+};
 
 /* This does not take a reference on the handle, the caller has to do it,
  * e.g. through _starpu_attempt_to_submit_data_request_from_apps()
@@ -235,8 +277,8 @@ void _starpu_display_msi_stats(void);
  * should thus *not* take a reference since it can not know whether the request will complete
  * async means that _starpu_fetch_data_on_node will wait for completion of the request
  */
-int _starpu_fetch_data_on_node(starpu_data_handle_t handle, struct _starpu_data_replicate *replicate,
-			       enum starpu_data_access_mode mode, unsigned detached, unsigned async,
+int _starpu_fetch_data_on_node(starpu_data_handle_t handle, int node, struct _starpu_data_replicate *replicate,
+			       enum starpu_data_access_mode mode, unsigned detached, unsigned is_prefetch, unsigned async,
 			       void (*callback_func)(void *), void *callback_arg);
 /* This releases a reference on the handle */
 void _starpu_release_data_on_node(struct _starpu_data_state *state, uint32_t default_wt_mask,
@@ -252,12 +294,15 @@ size_t _starpu_data_get_size(starpu_data_handle_t handle);
 
 uint32_t _starpu_data_get_footprint(starpu_data_handle_t handle);
 
-void _starpu_push_task_output(struct _starpu_job *j, uint32_t mask);
+void __starpu_push_task_output(struct _starpu_job *j);
+/* Version with driver trace */
+void _starpu_push_task_output(struct _starpu_job *j);
 
 void _starpu_release_nowhere_task_output(struct _starpu_job *j);
 
 STARPU_ATTRIBUTE_WARN_UNUSED_RESULT
-int _starpu_fetch_task_input(struct _starpu_job *j, uint32_t mask);
+int _starpu_fetch_task_input(struct _starpu_job *j);
+void _starpu_fetch_nowhere_task_input(struct _starpu_job *j);
 
 unsigned _starpu_is_data_present_or_requested(struct _starpu_data_state *state, unsigned node);
 
