@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2012                                     Inria
  * Copyright (C) 2012-2015,2017                           CNRS
- * Copyright (C) 2012-2014,2016-2017                      Université de Bordeaux
+ * Copyright (C) 2012-2014,2016-2018                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -148,6 +148,20 @@ free_coo_buffer_on_node(void *data_interface, unsigned node)
 	starpu_free_on_node(node, coo_interface->values, n_values * elemsize);
 }
 
+static int
+coo_pointer_is_inside(void *data_interface, unsigned node, void *ptr)
+{
+	struct starpu_coo_interface *coo_interface = (struct starpu_coo_interface *) data_interface;
+	(void) node;
+
+	return ((char*) ptr >= (char*) coo_interface->columns &&
+		(char*) ptr < (char*) coo_interface->columns + coo_interface->n_values * sizeof(coo_interface->columns[0]))
+	    || ((char*) ptr >= (char*) coo_interface->rows &&
+		(char*) ptr < (char*) coo_interface->rows + coo_interface->n_values * sizeof(coo_interface->rows[0]))
+	    || ((char*) ptr >= (char*) coo_interface->values &&
+		(char*) ptr < (char*) coo_interface->values + coo_interface->n_values * coo_interface->elemsize);
+}
+
 static size_t
 coo_interface_get_size(starpu_data_handle_t handle)
 {
@@ -207,6 +221,7 @@ struct starpu_data_interface_ops starpu_interface_coo_ops =
 	.register_data_handle  = register_coo_handle,
 	.allocate_data_on_node = allocate_coo_buffer_on_node,
 	.handle_to_pointer     = NULL,
+	.pointer_is_inside     = coo_pointer_is_inside,
 	.free_data_on_node     = free_coo_buffer_on_node,
 	.copy_methods          = &coo_copy_data_methods,
 	.get_size              = coo_interface_get_size,
