@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011, 2014  Université de Bordeaux
- * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2011,2014-2016                      Université de Bordeaux
+ * Copyright (C) 2010-2011,2013,2015,2017                 CNRS
+ * Copyright (C) 2015                                     Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -35,9 +36,8 @@ static void find_data_path(struct _starpu_data_state *data, unsigned path[])
 	unsigned level; /* level is the distance between the node and the current node */
 	for (level = 0; level < depth; level++)
 	{
-		STARPU_ASSERT(data);
 		path[depth - level - 1] = current->sibling_index;
-		current = data->father_handle;
+		current = current->father_handle;
 	}
 }
 
@@ -87,19 +87,31 @@ static int _starpu_compar_handles(const struct _starpu_data_descr *descrA,
 			else
 				/* Only A writes, take it first */
 				return -1;
-		} else
+		}
+		else
 			/* A doesn't write, take B before */
 			return 1;
 	}
 
+	/* Put arbitered accesses after non-arbitered */
+	if (dataA->arbiter && !(dataB->arbiter))
+		return 1;
+	if (dataB->arbiter && !(dataA->arbiter))
+		return -1;
+	if (dataA->arbiter != dataB->arbiter)
+		/* Both are arbitered, sort by arbiter pointer order */
+		return (dataA->arbiter < dataB->arbiter)?-1:1;
+	/* If both are arbitered by the same arbiter (or they are both not
+	 * arbitered), we'll sort them by handle */
+
 	/* In case we have data/subdata from different trees */
 	if (dataA->root_handle != dataB->root_handle)
-		return ((dataA->root_handle < dataB->root_handle)?-1:1);
+		return (dataA->root_handle < dataB->root_handle)?-1:1;
 
 	/* Things get more complicated: we need to find the location of dataA
 	 * and dataB within the tree. */
-	unsigned dataA_path[dataA->depth - 1];
-	unsigned dataB_path[dataB->depth - 1];
+	unsigned dataA_path[dataA->depth];
+	unsigned dataB_path[dataB->depth];
 
 	find_data_path(dataA, dataA_path);
 	find_data_path(dataB, dataB_path);
