@@ -3,7 +3,7 @@
  * Copyright (C) 2011-2012,2014                           Inria
  * Copyright (C) 2008-2017                                Université de Bordeaux
  * Copyright (C) 2010                                     Mehdi Juhoor
- * Copyright (C) 2010-2017                                CNRS
+ * Copyright (C) 2010-2018                                CNRS
  * Copyright (C) 2011                                     Télécom-SudParis
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -53,12 +53,12 @@ static int ncudagpus = -1;
 static size_t global_mem[STARPU_MAXCUDADEVS];
 #ifdef STARPU_USE_CUDA
 static cudaStream_t streams[STARPU_NMAXWORKERS];
+static char used_stream[STARPU_NMAXWORKERS];
 static cudaStream_t out_transfer_streams[STARPU_MAXCUDADEVS];
 static cudaStream_t in_transfer_streams[STARPU_MAXCUDADEVS];
 /* Note: streams are not thread-safe, so we define them for each CUDA worker
  * emitting a GPU-GPU transfer */
 static cudaStream_t in_peer_transfer_streams[STARPU_MAXCUDADEVS][STARPU_MAXCUDADEVS];
-static cudaStream_t out_peer_transfer_streams[STARPU_MAXCUDADEVS][STARPU_MAXCUDADEVS];
 static struct cudaDeviceProp props[STARPU_MAXCUDADEVS];
 #ifndef STARPU_SIMGRID
 static cudaEvent_t task_events[STARPU_NMAXWORKERS][STARPU_MAX_PIPELINE];
@@ -161,18 +161,11 @@ cudaStream_t starpu_cuda_get_local_out_transfer_stream()
 
 cudaStream_t starpu_cuda_get_peer_transfer_stream(unsigned src_node, unsigned dst_node)
 {
-	int worker = starpu_worker_get_id();
-	int devid = starpu_worker_get_devid(worker);
 	int src_devid = _starpu_memory_node_get_devid(src_node);
 	int dst_devid = _starpu_memory_node_get_devid(dst_node);
 	cudaStream_t stream;
 
-	STARPU_ASSERT(devid == src_devid || devid == dst_devid);
-
-	if (devid == dst_devid)
-		stream = in_peer_transfer_streams[src_devid][dst_devid];
-	else
-		stream = out_peer_transfer_streams[src_devid][dst_devid];
+	stream = in_peer_transfer_streams[src_devid][dst_devid];
 	STARPU_ASSERT(stream);
 	return stream;
 }
@@ -181,6 +174,7 @@ cudaStream_t starpu_cuda_get_local_stream(void)
 {
 	int worker = starpu_worker_get_id();
 
+	used_stream[worker] = 1;
 	return streams[worker];
 }
 
@@ -313,9 +307,6 @@ static void init_device_context(unsigned devid)
 		cures = starpu_cudaStreamCreate(&in_peer_transfer_streams[i][devid]);
 		if (STARPU_UNLIKELY(cures))
 			STARPU_CUDA_REPORT_ERROR(cures);
-		cures = starpu_cudaStreamCreate(&out_peer_transfer_streams[devid][i]);
-		if (STARPU_UNLIKELY(cures))
-			STARPU_CUDA_REPORT_ERROR(cures);
 	}
 }
 #endif /* !STARPU_SIMGRID */
@@ -358,7 +349,6 @@ static void deinit_device_context(unsigned devid)
 	for (i = 0; i < ncudagpus; i++)
 	{
 		cudaStreamDestroy(in_peer_transfer_streams[i][devid]);
-		cudaStreamDestroy(out_peer_transfer_streams[devid][i]);
 	}
 }
 #endif /* !STARPU_SIMGRID */
@@ -530,6 +520,14 @@ static void execute_job_on_cuda(struct starpu_task *task, struct _starpu_worker
 		}
 	}
 
+#ifndef STARPU_SIMGRID
+	if (!used_stream[workerid])
+	{
+		used_stream[workerid] = 1;
+		_STARPU_DISP("Warning: starpu_cuda_get_local_stream() was not used to submit kernel to CUDA on worker %d. CUDA will thus introduce a lot of useless synchronizations, which will prevent proper overlapping of data transfers and kernel execution. See the CUDA-specific part of the 'Check List When Performance Are Not There' of the StarPU handbook\n", workerid);
+	}
+#endif
+
 	if (task->cl->cuda_flags[j->nimpl] & STARPU_CUDA_ASYNC)
 	{
 		if (worker->pipeline_length == 0)
