@@ -1,7 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013  INRIA
- * Copyright (C) 2013  Simon Archipoff
+ * Copyright (C) 2013                                     Inria
+ * Copyright (C) 2014-2015,2017                           CNRS
+ * Copyright (C) 2014-2018                                Université de Bordeaux
+ * Copyright (C) 2013                                     Simon Archipoff
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,7 +21,6 @@
 #include <starpu_scheduler.h>
 
 #include "fifo_queues.h"
-#include "sched_component.h"
 
 struct _starpu_fifo_data
 {
@@ -209,7 +210,7 @@ static int fifo_can_push(struct starpu_sched_component * component)
 	STARPU_ASSERT(component->nchildren == 1);
 	struct starpu_sched_component * child = component->children[0];
 
-	struct starpu_task * task = starpu_sched_component_pull_task(component,NULL);
+	struct starpu_task * task = fifo_pull_task(component);
 	if(task)
 		ret = starpu_sched_component_push_task(NULL,child,task);	
 	while(task && !ret) 
@@ -217,7 +218,7 @@ static int fifo_can_push(struct starpu_sched_component * component)
 		if(!res)
 			res = 1;
 
-		task = starpu_sched_component_pull_task(component,NULL);
+		task = fifo_pull_task(component);
 		if(task)
 			ret = starpu_sched_component_push_task(NULL,child,task);	
 	}
