@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2012  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014  Centre National de la Recherche Scientifique
+ * Copyright (C) 2012-2013                                Inria
+ * Copyright (C) 2010-2015,2017                           CNRS
+ * Copyright (C) 2009-2012,2014,2017                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +23,7 @@
 #include <sys/time.h>
 
 #include <starpu.h>
-#include <config.h>
+#include <common/config.h>
 
 #include "starpufft.h"
 #ifdef STARPU_USE_CUDA
@@ -153,7 +154,7 @@ compute_roots(STARPUFFT(plan) plan)
 		plan->roots[dim] = malloc(plan->n[dim] * sizeof(**plan->roots));
 		for (k = 0; k < plan->n[dim]; k++)
 			plan->roots[dim][k] = cexp(exp*k);
-		starpu_vector_data_register(&plan->roots_handle[dim], 0, (uintptr_t) plan->roots[dim], plan->n[dim], sizeof(**plan->roots));
+		starpu_vector_data_register(&plan->roots_handle[dim], STARPU_MAIN_RAM, (uintptr_t) plan->roots[dim], plan->n[dim], sizeof(**plan->roots));
 
 #ifdef STARPU_USE_CUDA
 		if (plan->n[dim] > 100000)
@@ -174,7 +175,7 @@ compute_roots(STARPUFFT(plan) plan)
 
 /* Only CUDA capability >= 1.3 supports doubles, rule old card out.  */
 #ifdef DOUBLE
-static int can_execute(unsigned workerid, struct starpu_task *task, unsigned nimpl) {
+static int can_execute(unsigned workerid, struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, unsigned nimpl STARPU_ATTRIBUTE_UNUSED) {
 	if (starpu_worker_get_type(workerid) == STARPU_CPU_WORKER)
 		return 1;
 #ifdef STARPU_USE_CUDA
@@ -215,9 +216,9 @@ STARPUFFT(start)(STARPUFFT(plan) plan, void *_in, void *_out)
 			switch (plan->type)
 			{
 			case C2C:
-				starpu_vector_data_register(&plan->in_handle, 0, (uintptr_t) plan->in, plan->totsize, sizeof(STARPUFFT(complex)));
+				starpu_vector_data_register(&plan->in_handle, STARPU_MAIN_RAM, (uintptr_t) plan->in, plan->totsize, sizeof(STARPUFFT(complex)));
 				if (!PARALLEL)
-					starpu_vector_data_register(&plan->out_handle, 0, (uintptr_t) plan->out, plan->totsize, sizeof(STARPUFFT(complex)));
+					starpu_vector_data_register(&plan->out_handle, STARPU_MAIN_RAM, (uintptr_t) plan->out, plan->totsize, sizeof(STARPUFFT(complex)));
 				if (PARALLEL)
 				{
 					for (z = 0; z < plan->totsize1; z++)
@@ -232,9 +233,9 @@ STARPUFFT(start)(STARPUFFT(plan) plan, void *_in, void *_out)
 			break;
 		}
 		case 2:
-			starpu_vector_data_register(&plan->in_handle, 0, (uintptr_t) plan->in, plan->totsize, sizeof(STARPUFFT(complex)));
+			starpu_vector_data_register(&plan->in_handle, STARPU_MAIN_RAM, (uintptr_t) plan->in, plan->totsize, sizeof(STARPUFFT(complex)));
 			if (!PARALLEL)
-				starpu_vector_data_register(&plan->out_handle, 0, (uintptr_t) plan->out, plan->totsize, sizeof(STARPUFFT(complex)));
+				starpu_vector_data_register(&plan->out_handle, STARPU_MAIN_RAM, (uintptr_t) plan->out, plan->totsize, sizeof(STARPUFFT(complex)));
 			if (PARALLEL)
 			{
 				for (z = 0; z < plan->totsize1; z++)
@@ -307,7 +308,8 @@ STARPUFFT(execute_handle)(STARPUFFT(plan) plan, starpu_data_handle_t in, starpu_
 void
 STARPUFFT(destroy_plan)(STARPUFFT(plan) plan)
 {
-	int workerid, dim, i;
+	unsigned workerid;
+	int dim, i;
 
 	for (workerid = 0; workerid < starpu_worker_get_count(); workerid++)
 	{
@@ -436,7 +438,7 @@ STARPUFFT(free)(void *p)
 void
 STARPUFFT(showstats)(FILE *out)
 {
-	int worker;
+	unsigned worker;
 	unsigned total;
 
 #define TIMING(begin,end) (double)((end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec))
