@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011, 2013  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2011-2013                                Inria
+ * Copyright (C) 2010-2013,2015                           CNRS
+ * Copyright (C) 2010-2011,2013-2014,2016                 Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,19 +19,24 @@
 #include <starpu.h>
 #include "../helper.h"
 
+/*
+ * Trigger a lot of transfers of a single variable between CPUs and GPUs
+ */
+
 #ifdef STARPU_QUICK_CHECK
 static unsigned nloops = 10;
 #else
 static unsigned nloops = 1000;
 #endif
 
-static void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
+void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
 {
 }
 
 static struct starpu_codelet cpu_codelet =
 {
         .cpu_funcs = {dummy_func},
+		.cpu_funcs_name = {"dummy_func"},
         .model = NULL,
         .nbuffers = 1,
 	.modes = {STARPU_RW}
@@ -40,6 +46,7 @@ static struct starpu_codelet gpu_codelet =
 {
         .cuda_funcs = {dummy_func},
         .opencl_funcs = {dummy_func},
+		.cpu_funcs_name = {"dummy_func"},
         .model = NULL,
         .nbuffers = 1,
 	.modes = {STARPU_RW}
@@ -53,19 +60,20 @@ int main(int argc, char **argv)
         unsigned i;
 	int ret;
 
-        ret = starpu_init(NULL);
+        ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	if (starpu_worker_get_count_by_type(STARPU_CUDA_WORKER) == 0 && starpu_worker_get_count_by_type(STARPU_OPENCL_WORKER) == 0)
+	if (starpu_worker_get_count_by_type(STARPU_CUDA_WORKER) == 0 && starpu_worker_get_count_by_type(STARPU_OPENCL_WORKER) == 0 &&
+		starpu_worker_get_count_by_type(STARPU_MIC_WORKER) == 0)
 	{
-		FPRINTF(stderr, "This application requires a CUDA or OpenCL Worker\n");
+		FPRINTF(stderr, "This application requires a CUDA , OpenCL or MIC Worker\n");
 		starpu_shutdown();
 		return STARPU_TEST_SKIPPED;
 	}
 
         foo = 0.0f;
-	starpu_variable_data_register(&float_array_handle, 0, (uintptr_t)&foo, sizeof(foo));
+	starpu_variable_data_register(&float_array_handle, STARPU_MAIN_RAM, (uintptr_t)&foo, sizeof(foo));
 
         for (i = 0; i < nloops; i++)
 	{
