@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2010-2011,2013-2014,2016                 Université de Bordeaux
+ * Copyright (C) 2011-2013,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,7 +16,6 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
@@ -23,14 +23,16 @@
 #include <stdlib.h>
 #include "../helper.h"
 
+/*
+ * Use a void interface to protect the access to a variable that is not declared to StarPU
+ */
+
 starpu_data_handle_t void_handle;
 
 int critical_var;
 
-static void critical_section(void *descr[], __attribute__ ((unused)) void *_args)
+void critical_section(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
-	STARPU_SKIP_IF_VALGRIND;
-
 	/* We do not protect this variable because it is only accessed when the
 	 * "void_handle" piece of data is accessed. */
 	critical_var++;
@@ -38,23 +40,23 @@ static void critical_section(void *descr[], __attribute__ ((unused)) void *_args
 
 static struct starpu_codelet cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {critical_section, NULL},
-	.cuda_funcs = {critical_section, NULL},
-	.opencl_funcs = {critical_section, NULL},
+	.cpu_funcs = {critical_section},
+	.cuda_funcs = {critical_section},
+	.opencl_funcs = {critical_section},
 	.nbuffers = 1,
 	.modes = {STARPU_RW}
 };
 
 int main(int argc, char **argv)
 {
+#ifdef STARPU_QUICK_CHECK
+	int ntasks = 10;
+#else
 	int ntasks = 1000;
-	int ret;
-
-#ifdef STARPU_SLOW_MACHINE
-	ntasks /= 10;
 #endif
 
+	int ret;
+
 	ret = starpu_init(NULL);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
@@ -82,7 +84,7 @@ int main(int argc, char **argv)
 
 	starpu_shutdown();
 
-	STARPU_RETURN(ret);
+	return ret;
 
 enodev:
 	fprintf(stderr, "WARNING: No one can execute this task\n");
