@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011-2013                                Inria
+ * Copyright (C) 2012-2013,2015,2017                      CNRS
+ * Copyright (C) 2013                                     Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -13,7 +15,7 @@
  *
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
-#include <config.h>
+
 #include <starpu.h>
 #include "../test_interfaces.h"
 #include "../../../helper.h"
@@ -25,7 +27,7 @@
 /* Prototypes */
 static void register_data(void);
 static void unregister_data(void);
-static void test_block_cpu_func(void *buffers[], void *args);
+void test_block_cpu_func(void *buffers[], void *args);
 #ifdef STARPU_USE_CUDA
 extern void test_block_cuda_func(void *buffers[], void *_args);
 #endif
@@ -33,7 +35,6 @@ extern void test_block_cuda_func(void *buffers[], void *_args);
 extern void test_block_opencl_func(void *buffers[], void *args);
 #endif
 
-
 static starpu_data_handle_t _block_handle;
 static starpu_data_handle_t _block2_handle;
 
@@ -45,6 +46,9 @@ struct test_config block_config =
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_func   = test_block_opencl_func,
+#endif
+#ifdef STARPU_USE_MIC
+	.cpu_func_name = "test_block_cpu_func",
 #endif
 	.handle        = &_block_handle,
 	.dummy_handle  = &_block2_handle,
@@ -68,7 +72,7 @@ register_data(void)
 
 	/* Registering data */
 	starpu_block_data_register(&_block_handle,
-                                    0,
+                                    STARPU_MAIN_RAM,
                                     (uintptr_t)_block,
 				    NX,
 				    NX * NY,
@@ -77,7 +81,7 @@ register_data(void)
 				    NZ,
 				    sizeof(_block[0]));
 	starpu_block_data_register(&_block2_handle,
-                                    0,
+                                    STARPU_MAIN_RAM,
                                     (uintptr_t)_block2,
 				    NX,
 				    NX * NY,
@@ -94,7 +98,7 @@ unregister_data(void)
 	starpu_data_unregister(_block2_handle);
 }
 
-static void test_block_cpu_func(void *buffers[], void *args)
+void test_block_cpu_func(void *buffers[], void *args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -130,15 +134,16 @@ static void test_block_cpu_func(void *buffers[], void *args)
 }
 
 int
-main(void)
+main(int argc, char **argv)
 {
 	data_interface_test_summary *summary;
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
 	conf.ncuda = 2;
 	conf.nopencl = 1;
+	conf.nmic = -1;
 
-	if (starpu_init(&conf) == -ENODEV || starpu_cpu_worker_get_count() == 0)
+	if (starpu_initialize(&conf, &argc, &argv) == -ENODEV || starpu_cpu_worker_get_count() == 0)
 		goto enodev;
 
 	register_data();
