@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011-2012                                Inria
+ * Copyright (C) 2012-2013,2015-2017                      CNRS
+ * Copyright (C) 2013-2014                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -14,7 +16,6 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <starpu.h>
 #include "generic.h"
 #include "../../../../helper.h"
@@ -73,6 +74,7 @@ void opencl_func(void *buffers[], void *args)
 	global_stats.opencl++;
 }
 
+static
 void cpu_to_opencl_func(void *buffers[], void *args)
 {
 	STARPU_SKIP_IF_VALGRIND;
@@ -80,6 +82,7 @@ void cpu_to_opencl_func(void *buffers[], void *args)
 	global_stats.cpu_to_opencl++;
 }
 
+static
 void opencl_to_cpu_func(void *buffers[], void *args)
 {
 	STARPU_SKIP_IF_VALGRIND;
@@ -100,6 +103,57 @@ struct starpu_codelet opencl_to_cpu_cl =
 };
 #endif /* !STARPU_USE_OPENCL */
 
+#ifdef STARPU_USE_MIC
+void mic_dummy_kernel(void *buffers[], void *cl_arg)
+{
+}
+
+starpu_mic_kernel_t mic_get_kernel()
+{
+	static starpu_mic_func_symbol_t mic_symbol = NULL;
+	if (mic_symbol == NULL)
+		starpu_mic_register_kernel(&mic_symbol, "mic_dummy_kernel");
+
+	return starpu_mic_get_kernel(mic_symbol);
+}
+
+starpu_mic_kernel_t mic_func()
+{
+	STARPU_SKIP_IF_VALGRIND;
+
+	global_stats.mic++;
+
+	return mic_get_kernel();
+}
+
+starpu_mic_kernel_t cpu_to_mic_func()
+{
+	STARPU_SKIP_IF_VALGRIND;
+
+	global_stats.cpu_to_mic++;
+
+	return mic_get_kernel();
+}
+
+void mic_to_cpu_func(void *buffers[], void *args)
+{
+	STARPU_SKIP_IF_VALGRIND;
+
+	global_stats.mic_to_cpu++;
+}
+
+struct starpu_codelet cpu_to_mic_cl =
+{
+	.mic_funcs = {cpu_to_mic_func},
+	.nbuffers = 1
+};
+
+struct starpu_codelet mic_to_cpu_cl =
+{
+	.cpu_funcs = {mic_to_cpu_func},
+	.nbuffers = 1
+};
+#endif // STARPU_USE_MIC
 
 struct starpu_multiformat_data_interface_ops ops =
 {
@@ -112,6 +166,11 @@ struct starpu_multiformat_data_interface_ops ops =
 	.opencl_elemsize = sizeof(int),
 	.cpu_to_opencl_cl = &cpu_to_opencl_cl,
 	.opencl_to_cpu_cl = &opencl_to_cpu_cl,
+#endif
+#ifdef STARPU_USE_MIC
+	.mic_elemsize = sizeof(int),
+	.cpu_to_mic_cl = &cpu_to_mic_cl,
+	.mic_to_cpu_cl = &mic_to_cpu_cl,
 #endif
 	.cpu_elemsize = sizeof(int)
 };
@@ -138,6 +197,14 @@ print_stats(struct stats *s)
 			s->cpu_to_opencl,
 			s->opencl_to_cpu);
 #endif /* !STARPU_USE_OPENCL */
+#ifdef STARPU_USE_MIC
+	FPRINTF(stderr, "mic	: %u\n"
+			"cpu->mic : %u\n"
+			"mic->cpu : %u\n",
+			s->mic,
+			s->cpu_to_mic,
+			s->mic_to_cpu);
+#endif
 }
 
 void reset_stats(struct stats *s)
@@ -155,6 +222,11 @@ void reset_stats(struct stats *s)
 	s->cpu_to_opencl = 0;
 	s->opencl_to_cpu = 0;
 #endif /* !STARPU_USE_OPENCL */
+#ifdef STARPU_USE_MIC
+	s->mic = 0;
+	s->cpu_to_mic = 0;
+	s->mic_to_cpu = 0;
+#endif
 }
 
 int
@@ -174,6 +246,11 @@ compare_stats(struct stats *s1, struct stats *s2)
 	    s1->cpu_to_opencl == s2->cpu_to_opencl &&
 	    s1->opencl_to_cpu == s2->opencl_to_cpu &&
 #endif /* !STARPU_USE_OPENCL */
+#ifdef STARPU_USE_MIC
+		s1->mic == s2->mic &&
+		s1->cpu_to_mic == s2->cpu_to_mic &&
+		s1->mic_to_cpu == s2->mic_to_cpu &&
+#endif
 	    1 /* Just so the build does not fail if we disable EVERYTHING */
 	)
 		return 0;
