@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011-2013                                Inria
+ * Copyright (C) 2012-2013,2015,2017                      CNRS
+ * Copyright (C) 2014,2016                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -46,8 +48,7 @@ test(void)
 
 	static struct starpu_codelet cl_cuda =
 	{
-		.where     = STARPU_CUDA,
-		.cuda_funcs = {cuda_func, NULL},
+		.cuda_funcs = {cuda_func},
 		.nbuffers  = 1,
 		.modes = {STARPU_RW}
 	};
@@ -57,12 +58,15 @@ test(void)
 	task_cuda->handles[0] = handle;
 	ret = starpu_task_submit(task_cuda);
 	if (ret != 0)
+	{
+		task_cuda->destroy = 0;
+		starpu_task_destroy(task_cuda);
 		return 1;
+	}
 
 	static struct starpu_codelet cl_opencl =
 	{
-		.where       = STARPU_OPENCL,
-		.opencl_funcs = {opencl_func, NULL},
+		.opencl_funcs = {opencl_func},
 		.nbuffers    = 1,
 		.modes = {STARPU_RW}
 	};
@@ -72,14 +76,22 @@ test(void)
 	task_opencl->handles[0] = handle;
 	ret = starpu_task_submit(task_opencl);
 	if (ret != 0)
+	{
+		task_opencl->destroy = 0;
+		starpu_task_destroy(task_opencl);
 		return 1;
+	}
 
 	task_cuda2 = starpu_task_create();
 	task_cuda2->cl = &cl_cuda;
 	task_cuda2->handles[0] = handle;
 	ret = starpu_task_submit(task_cuda2);
 	if (ret != 0)
+	{
+		task_cuda2->destroy = 0;
+		starpu_task_destroy(task_cuda2);
 		return 1;
+	}
 
 	return 0;
 }
@@ -90,7 +102,7 @@ register_handle(void)
 	int i;
 	for (i = 0; i < NX; i++)
 		vector[i] = i;
-	starpu_multiformat_data_register(&handle, 0, vector, NX, &ops);
+	starpu_multiformat_data_register(&handle, STARPU_MAIN_RAM, vector, NX, &ops);
 }
 
 static void
@@ -105,12 +117,11 @@ main(void)
 {
 #if defined(STARPU_USE_CUDA) && defined(STARPU_USE_OPENCL)
 	int ret;
-	struct starpu_conf conf =
-	{
-		.ncpus   = -1,
-		.ncuda   = 1,
-		.nopencl = 1
-	};
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
+
+	conf.ncuda = 1;
+	conf.nopencl = 1;
 
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV)
