@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  INRIA
- * Copyright (C) 2013, 2014  CNRS
+ * Copyright (C) 2011-2013                                Inria
+ * Copyright (C) 2012-2015,2017                           CNRS
+ * Copyright (C) 2013,2015, 2018                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -87,6 +88,7 @@ struct data_interface_test_summary
 	int compare;
 #ifdef STARPU_USE_CPU
 	int handle_to_pointer;
+	int pointer_is_inside;
 #endif
 };
 
@@ -146,6 +148,8 @@ void data_interface_test_summary_print(FILE *f,
 		enum_to_string(s->cpu_to_cpu));
 	FPRINTF(f, "handle_to_pointer() : %s\n",
 		enum_to_string(s->handle_to_pointer));
+	FPRINTF(f, "pointer_is_inside() : %s\n",
+		enum_to_string(s->pointer_is_inside));
 #endif /* !STARPU_USE_CPU */
 	FPRINTF(f, "compare()        : %s\n",
 		enum_to_string(s->compare));
@@ -267,6 +271,7 @@ static struct data_interface_test_summary summary =
 #endif
 #ifdef STARPU_USE_CPU
 	.handle_to_pointer     = UNTESTED,
+	.pointer_is_inside     = UNTESTED,
 #endif
 	.success               = SUCCESS
 };
@@ -457,7 +462,7 @@ ram_to_cuda(void)
 	return current_config->copy_failed;
 }
 
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 static enum exit_code
 cuda_to_cuda(void)
 {
@@ -585,7 +590,7 @@ run_cuda(int async)
 	if (err != SUCCESS)
 		return;
 
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 	if (starpu_cuda_worker_get_count() >= 2)
 	{
 		err = cuda_to_cuda();
@@ -599,7 +604,7 @@ run_cuda(int async)
 	}
 #else
 	summary.cuda_to_cuda_async = UNTESTED;
-#endif /* !HAVE_CUDA_MEMCPY_PEER */
+#endif /* !STARPU_HAVE_CUDA_MEMCPY_PEER */
 
 #ifdef STARPU_USE_CPU
 	err = cuda_to_ram();
@@ -807,6 +812,43 @@ handle_to_pointer(void)
 	if (tests > 0)
 		summary.handle_to_pointer = SUCCESS;
 }
+
+static void
+pointer_is_inside(void)
+{
+	void *ptr;
+	unsigned int node;
+	unsigned int tests = 0;
+	starpu_data_handle_t handle;
+
+	handle = *current_config->handle;
+	if (!handle->ops->pointer_is_inside || !handle->ops->handle_to_pointer)
+		return;
+
+	for (node = 0; node < STARPU_MAXNODES; node++)
+	{
+		if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
+			continue;
+		if (!starpu_data_test_if_allocated_on_node(handle, node))
+			continue;
+
+		ptr = handle->ops->handle_to_pointer(handle, node);
+		if (starpu_data_lookup(ptr) != handle)
+		{
+			summary.pointer_is_inside = FAILURE;
+			return;
+		}
+		if (!starpu_data_pointer_is_inside(handle, node, ptr))
+		{
+			summary.pointer_is_inside = FAILURE;
+			return;
+		}
+		tests++;
+	}
+
+	if (tests > 0)
+		summary.pointer_is_inside = SUCCESS;
+}
 #endif /* !STARPU_USE_CPU */
 
 static int
@@ -851,6 +893,7 @@ run_tests(struct test_config *conf)
 	ram_to_ram();
 	compare();
 	handle_to_pointer();
+	pointer_is_inside();
 #endif
 
 	return &summary;
