@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011-2012                                Inria
+ * Copyright (C) 2012,2015-2017                           CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -13,9 +14,8 @@
  *
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
-#include <config.h>
+
 #include <starpu.h>
-#include <starpu_opencl.h>
 #include "../test_interfaces.h"
 
 #define KERNEL_LOCATION "tests/datawizard/interfaces/vector/test_vector_opencl_kernel.cl"
@@ -42,7 +42,7 @@ test_vector_opencl_func(void *buffers[], void *args)
 	cl_mem val = (cl_mem)STARPU_VECTOR_GET_DEV_HANDLE(buffers[0]);
 
 	cl_context context;
-	id = starpu_worker_get_id();
+	id = starpu_worker_get_id_check();
 	devid = starpu_worker_get_devid(id);
 	starpu_opencl_get_context(devid, &context);
 
