@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Université de Bordeaux
+ * Copyright (C) 2012-2013                                Inria
+ * Copyright (C) 2012-2013,2015                           CNRS
+ * Copyright (C) 2011,2013-2014,2016                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,15 +20,22 @@
 #include "../helper.h"
 #include "scal.h"
 
+/*
+ * Test partitioning an uninitialized vector
+ */
+
 struct starpu_codelet mycodelet =
 {
-	.cpu_funcs = { scal_func_cpu},
+	.cpu_funcs = { scal_func_cpu },
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = { scal_func_opencl},
+	.opencl_funcs = { scal_func_opencl },
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = { scal_func_cuda},
+	.cuda_funcs = { scal_func_cuda },
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
+	.cpu_funcs_name = {"scal_func_cpu"},
 	.modes = { STARPU_W },
         .model = NULL,
         .nbuffers = 1
@@ -40,7 +49,7 @@ int main(int argc, char **argv)
 	int n, size;
 	unsigned i;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -64,7 +73,7 @@ int main(int argc, char **argv)
 
 	for (i = 0; i < f.nchildren; i++)
 	{
-		ret = starpu_insert_task(&mycodelet,
+		ret = starpu_task_insert(&mycodelet,
 					 STARPU_W,
 					 starpu_data_get_sub_data(handle, 1, i),
 					 0);
@@ -75,7 +84,7 @@ int main(int argc, char **argv)
 	ret = starpu_task_wait_for_all();
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_wait_for_all");
 
-	starpu_data_unpartition(handle, 0);
+	starpu_data_unpartition(handle, STARPU_MAIN_RAM);
 	starpu_data_unregister(handle);
 	starpu_shutdown();
 
