@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2013  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2010-2013,2015                           CNRS
+ * Copyright (C) 2010,2013-2014,2016                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,7 +22,11 @@
 #include <starpu.h>
 #include "../helper.h"
 
-static void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
+/*
+ * Test task dependencies and waiting for a task
+ */
+
+void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
 {
 }
 
@@ -30,6 +35,7 @@ static struct starpu_codelet dummy_codelet =
 	.cpu_funcs = {dummy_func},
 	.cuda_funcs = {dummy_func},
 	.opencl_funcs = {dummy_func},
+	.cpu_funcs_name = {"dummy_func"},
         .model = NULL,
 	.nbuffers = 0
 };
@@ -49,7 +55,7 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
