@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012, 2014-2015  Université de Bordeaux
- * Copyright (C) 2012, 2013  Centre National de la Recherche Scientifique
- * Copyright (C) 2012 inria
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2011-2016                                Université de Bordeaux
+ * Copyright (C) 2011-2013,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,10 +16,13 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <starpu.h>
 #include "../helper.h"
 
+/*
+ * Benchmark memset with a non-linear regression
+ */
+
 #define START_LOG 8
 #ifdef STARPU_QUICK_CHECK
 #define END_LOG 20
@@ -36,11 +39,10 @@ static void memset_cuda(void *descr[], void *arg)
 	unsigned n = STARPU_VECTOR_GET_NX(descr[0]);
 
 	cudaMemsetAsync(ptr, 42, n * sizeof(*ptr), starpu_cuda_get_local_stream());
-	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 }
 #endif
 
-static void memset_cpu(void *descr[], void *arg)
+void memset_cpu(void *descr[], void *arg)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -64,11 +66,14 @@ static struct starpu_codelet memset_cl =
 {
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {memset_cuda},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {memset_opencl},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
 	.cpu_funcs = {memset_cpu},
+	.cpu_funcs_name = {"memset_cpu"},
 	.model = &model,
 	.nbuffers = 1,
 	.modes = {STARPU_W}
@@ -112,7 +117,7 @@ int main(int argc, char **argv)
 	conf.sched_policy_name = "eager";
 	conf.calibrate = 2;
 
-	ret = starpu_init(&conf);
+	ret = starpu_initialize(&conf, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
